% =======================  Supplementray code ======================= 
% Manuscript title:
% "Modelling the downstream development of a turbulent boundary layer 
% following a step change of roughness"
% by M. Li, C. M. de Silva, D. Chung, D. I. Pullin, I. Marusic and N.
% Hutchins
%
% Version: May 2022
% ===================================================================


% -------------------- Input parameters --------------------
% deltaplus0, ksplus
% (Uinfty, ks1, ks2): no effect on nondimensional results
% ----------------------------------------------------------

% --------------------- User settings ----------------------
% opt.Case, opt.Model
% ----------------------------------------------------------




clear; close all; clc;

PI = 0.56;         kappa = 0.384;     
A = 4.17;          B = 8.5;
Uinfty = 20;       nu = 1.5e-5;

% shear stress profile parameters (see equation (4.7))
p1 = 1.9971;       p2 = -3.0789;

deltaplus0 = 16500;  

% options
opt.Case = 'RtoS';
opt.Model = 'FTBL';  % 'E58'; 'FTBL'; 'FTBL-Blend'

if strcmp(opt.Case,'RtoS')
    % =========== R to S ===============
    ks1 = 2.43e-3; 
    ks2 = 0;
    ksplus = 160;
    utau1 = ksplus*nu/ks1;
    ks1plus = ksplus;
    ks2plus = 0;
    Hist.r2(1) = 0.6;  % r2 = utau2/utau0
    Hist.r1(1) = 1;    % r1 = utau1/utau0

elseif strcmp(opt.Case,'StoR')
    % =========== S to R ===============
    ks2 = 2.43e-3;  
    ks1 = 0;
    ksplus = 160;
    utau1 = ksplus*nu/ks2;
    ks1plus = 0;
    ks2plus = ksplus;
    Hist.r2(1) = 100;  % r2 = utau2/utau0
    Hist.r1(1) = 1;   % r1 = utau1/utau0
end


if strcmp(opt.Model,'E58')
    opt.Wake = 0;
    opt.ShearCorr = 0;
    opt.DeltaGrow = 0;
    opt.Blend = 0;
    opt.Compo = 0;
elseif strcmp(opt.Model,'FTBL')
    opt.Wake = 1;
    opt.ShearCorr = 1;
    opt.DeltaGrow = 1;
    opt.Blend = 0;
    opt.Compo = 0;
elseif strcmp(opt.Model,'FTBL-Blend')
    opt.Wake = 1;
    opt.ShearCorr = 1;
    opt.DeltaGrow = 1;
    opt.Blend = 1;
    opt.Compo = 1;
end
    
Bparam.DIBL = 500;
Bparam.mu0 = 0.7;
Bparam.sigma = 1;

if opt.Blend == 0
    Bparam.DIBL = 0;
end

Const.PI = PI;
Const.kappa = kappa;
Const.A = A;
Const.B = B;
Const.nu = nu;
Const.p1 = p1;
Const.p2 = p2;

z01 = max(ks1*exp(-kappa*B),nu*exp(-kappa*A)/(Uinfty/30));
z02 = max(ks2*exp(-kappa*B),nu*exp(-kappa*A)/(Uinfty/30));
DeltaUplus1 = roughness_function(ks1plus, kappa, A, B);
[Uplusf,zplus] = Uplus_profile(deltaplus0,opt.Wake*PI,kappa,A,DeltaUplus1);
Uinftyplus = max(Uplusf);

Condi.ks1plus = ks1plus;
Condi.ks2plus = ks2plus;
Condi.Uinftplus = Uinftyplus;
Condi.deltaplus0 = deltaplus0;

NonVar.Condi = Condi;
NonVar.Const = Const;
NonVar.opt = opt;
NonVar.Bparam = Bparam;


eps = 0.5e-6;

Hist.IBLplus_range = logspace(log10(max(Bparam.DIBL,0)+max(z01,z02)*utau1/nu+5),log10(deltaplus0*5),1000);

for loop = 1:numel(Hist.IBLplus_range)
    IBLplus = Hist.IBLplus_range(loop);
    disp(loop)
    if loop == 1
        
        % initialise first step
        r1 = 1;
        deltaplus = deltaplus0;
        funcS = @(r2_) fun1(r1,r2_,deltaplus,IBLplus,NonVar);
        r2 = fzero(funcS,Hist.r2(1));
        
    elseif loop >1
        
        if ~opt.DeltaGrow
            
            funcS = @(r2_) fun1(r1,r2_,deltaplus,IBLplus,NonVar);
            r2 = fzero(funcS,Hist.r2(loop-1));
            
        elseif opt.DeltaGrow
            i = 1;  % count of iterations

            % initialise subsequent steps using values from the previous step
            f(1,1) = fun1(Hist.r1(loop-1),Hist.r2(loop-1),Hist.deltaplus(loop-1),IBLplus,NonVar);
            f(2,1) = fun2(Hist.r1(loop-1),Hist.deltaplus(loop-1),NonVar);
            f(3,1) = fun3(Hist.r1(loop-1),Hist.r2(loop-1),Hist.deltaplus(loop-1),IBLplus,loop,Hist,NonVar);
            y(1,1) = Hist.r1(loop-1);
            y(2,1) = Hist.r2(loop-1);
            y(3,1) = Hist.deltaplus(loop-1);

            while norm(f,2)>eps
                i = i+1;

                % calculate Jacobian matrix
                Dmat(1,1) = (fun1(r1*(1+eps),r2,deltaplus,IBLplus,NonVar)-fun1(r1*(1-eps),r2,deltaplus,IBLplus,NonVar))/(2*r1*eps);
                Dmat(1,2) = (fun1(r1,r2*(1+eps),deltaplus,IBLplus,NonVar)-fun1(r1,r2*(1-eps),deltaplus,IBLplus,NonVar))/(2*r2*eps);
                Dmat(1,3) = (fun1(r1,r2,deltaplus*(1+eps),IBLplus,NonVar)-fun1(r1,r2,deltaplus*(1-eps),IBLplus,NonVar))/(2*deltaplus*eps);
                Dmat(2,1) = (fun2(r1*(1+eps),deltaplus,NonVar)-fun2(r1*(1-eps),deltaplus,NonVar))/(2*r1*eps);
                Dmat(2,2) = 0;
                Dmat(2,3) = (fun2(r1,deltaplus*(1+eps),NonVar)-fun2(r1,deltaplus*(1-eps),NonVar))/(2*deltaplus*eps);
                Dmat(3,1) = (fun3(r1*(1+eps),r2,deltaplus,IBLplus,loop,Hist,NonVar)-fun3(r1*(1-eps),r2,deltaplus,IBLplus,loop,Hist,NonVar))/(2*r1*eps);
                Dmat(3,2) = (fun3(r1,r2*(1+eps),deltaplus,IBLplus,loop,Hist,NonVar)-fun3(r1,r2*(1-eps),deltaplus,IBLplus,loop,Hist,NonVar))/(2*r2*eps);
                Dmat(3,3) = (fun3(r1,r2,deltaplus*(1+eps),IBLplus,loop,Hist,NonVar)-fun3(r1,r2,deltaplus*(1-eps),IBLplus,loop,Hist,NonVar))/(2*deltaplus*eps);

                f(1,1) = fun1(r1,r2,deltaplus,IBLplus,NonVar);
                f(2,1) = fun2(r1,deltaplus,NonVar);
                f(3,1) = fun3(r1,r2,deltaplus,IBLplus,loop,Hist,NonVar);

                y(:,i) = y(:,i-1)-Dmat\f;

                r1 = y(1,i);
                r2 = y(2,i);
                deltaplus = y(3,i);
            end  
        end
    end
    
    DeltaUplus1 = roughness_function(ks1plus*r1, kappa, A, B);
    DeltaUplus2 = roughness_function(ks2plus*r2, kappa, A, B);
    Bparam_local = Bparam;
    Bparam_local.DIBL = Bparam.DIBL*r1;

    [zplus,Uplusf,~,~] = NonD_VeloProf(deltaplus*r1,IBLplus*r1,r2/r1,DeltaUplus1,DeltaUplus2,Const,opt,Bparam_local);
    
        
    n = numel(zplus)/2;
    Hist.G1(loop) = trapz(zplus(1:n)/r1,Uplusf(1:n)*r1);
    Hist.G2(loop) = trapz(zplus(1:n)/r1,(Uplusf(1:n)*r1).^2);
    Hist.G3(loop) = trapz(zplus(n:end)/r1,Uplusf(n:end)*r1);
    Hist.G4(loop) = trapz(zplus(n:end)/r1,(Uplusf(n:end)*r1).^2);
    
    Hist.Ui(loop) = Uplusf(n)*r1;
    
    if opt.ShearCorr
        Hist.shear1(loop) = r1^2*(p1*(IBLplus/deltaplus)^3+p2*(IBLplus/deltaplus)^2+(-1-p1-p2)*(IBLplus/deltaplus)+1);
        Hist.shear2(loop) = r1^2*(p1*(IBLplus/deltaplus)^3+p2*(IBLplus/deltaplus)^2+(-1-p1-p2)*(IBLplus/deltaplus)+1)-r2^2;
    elseif ~opt.ShearCorr
        Hist.shear1(loop) = r1^2; 
        Hist.shear2(loop) = r1^2-r2^2; 
    end
    
    Hist.Cf2(loop) = 2*(r2/Uinftyplus)^2;
    Hist.thetaplus(loop) = trapz(zplus/r1,Uplusf*r1.*(Uinftyplus-Uplusf*r1)/Uinftyplus^2);
    if loop>1
        if ~opt.DeltaGrow
            Hist.xplus(loop) = trapz(Hist.G2(1:loop),1./Hist.shear2(1:loop))-...
            trapz(Hist.G1(1:loop),Hist.Ui(1:loop)./Hist.shear2(1:loop));
        elseif opt.DeltaGrow
            Hist.xplus(loop) = trapz(Hist.thetaplus(1:loop),2./Hist.Cf2(1:loop));
        end
    end
    Hist.r1(loop) = r1;
    Hist.r2(loop) = r2;
    Hist.deltaplus(loop) = deltaplus;
    
    % determine delta99 from velocity profile
    ind1 = find(Uplusf>0.8*max(Uplusf),1,'first');
    ind2 = find(Uplusf>0.999*max(Uplusf),1,'first');
    Hist.delta99plus(loop) = interp1(Uplusf(ind1:ind2)/max(Uplusf),zplus(ind1:ind2)/r1,0.99,'pchip');
    
    % terminate the simulation when delta_i/delta_c>0.95
    if IBLplus/deltaplus>0.95
        Hist.IBLplus_range = Hist.IBLplus_range(1:loop);
        break
    end
end

% plot results
figure;
plot(Hist.xplus,Hist.IBLplus_range); xlabel('xU_{\tau0}/\nu'); ylabel('\delta_iU_{\tau0}/\nu')
figure;
plot(Hist.xplus,Hist.r2); xlabel('xU_{\tau0}/\nu'); ylabel('U_{\tau2}/U_{\tau0}')

save([opt.Model,'_',opt.Case,'_deltaplus0',num2str(deltaplus0),'_ksplus',num2str(ksplus),'.mat'],'-struct','Hist')